#!/bin/bash
# shellcheck disable=SC2155,SC1090,SC2012,SC2181
#
# This is the application upgrade script, which supports
# upgrading the filesystem with application level packages
# (RPMs) or generic .run files generated by makeself.
#
# Users should call this script prior to launching their application
# script (which in turn is usually called by init.d).
#
# This script was designed to support init.d, so it should be
# called with a runtime argument "start" if it is instantiated
# directly instead of through the init.d framework.

# function die
# print an error message and exit with status
# arg 1 is the exit status
# args 2- are the message to print
die()
{
    rv=$1
    shift
    echo "$*" >&2
    exit "$rv"
}

CONFFILE=/etc/default/app-upgrade
. ${CONFFILE}

log_update() {
    local dt=$(date +"[%m-%d %H:%M:%S]")
    echo -e "${dt}\t$*" | tee -a "${UPDATE_LOG}"
}

# function check_diskspace
# return 0 if space ok or 1 if space limited
check_diskspace()
{
    local tfile=/tmp/diskmon$$.du
    df -lPH  -x udf -x nfs -x tmpfs -x iso9660 -x aufs | grep -vE '^Filesystem|tmpfs|cdrom' > $tfile
    df -PH /tmp|grep -v '^Fil' >> $tfile
    awk '{ print $5 " " $1 " " $6 }' $tfile | sort -u | while read -r output;
    do
        #echo $output
        max_allow=${ALERT}

        usep=$(echo "$output" | awk '{ print $1}' | cut -d'%' -f1  )
        partition=$(echo "$output" | awk '{ print $2 }' )
        mountpt=$(echo "$output" | awk '{ print $3 }' )
        [ -f "${mountpt}/.maxuse" ] && max_allow=$(awk '/^ALERT/ { print $2}' "${mountpt}/.maxuse")
        if [ "$usep" -ge "$max_allow" ]; then
            echo "Running out of space \"$partition $mountpt ($usep%)\" on $(hostname) as of $(date)"
        fi
    done
    rm -f "$tfile"
}

# check versions of update packages against installed versions
# parameter: name of ipk file to check
# returns 0 if same version 1 if not
# assumes that if opkg info on just the name of the package returns package info then package is installed maybe we should check installed status
check_version()
{
    local pkg_info="$(opkg info "$1")"
    local rev_file="$(echo "$pkg_info" | awk '$1=="Version:"{print $2}' )"
    local pkg_name="$(echo "$pkg_info" | awk '$1=="Package:"{print $2}' )"
    local installed_info="$(opkg info "$pkg_name"))"
    local installed_rev="$( echo "$installed_info" |awk '$1=="Version:"{print $2}' )"
    echo file version:"$rev_file" installed version: "$installed_rev"
    if [ "$rev_file" = "$installed_rev" ]
    then
        return 0
    else
        return 1
    fi
}

case $1 in
start)
    echo "app-upgrade -> checking $UPDATES_DIR"

    # if there is an updates folder, check it out
    if [ -d "$UPDATES_DIR" ]; then

        mkdir -p "$PREV_UPDATES_DIR" 2>/dev/null

        # if we are short on diskspace, remove old updates if possible
        while ! check_diskspace
        do
            fname=$(ls -1t "${PREV_UPDATES_DIR}"/*.{run,rpm,ipk} 2>/dev/null | tail -1)
            [ "x" == "x$fname" ] && break
            dt=$(date +"[%m-%d %H:%M:%S]")
            rm -f "${fname}"
            log_update "${fname} DELETED"
        done

        # check for .run files.  If we find them, execute them.
        cd "${UPDATES_DIR}" 2>/dev/null || die 1 "Couldn't access update folder"

        count=$(ls -l ./*.run 2>/dev/null | wc -l)
        if [ "$count" != 0 ]; then
            # Get files from oldest to newest
            files=$(ls -rt ./*.run)
            for f in $files; do
                log_update "$f"
                mv "$f" "$PREV_UPDATES_DIR"
                echo "Running update \"$f\" ..."
                chmod +x "$PREV_UPDATES_DIR/$f"
                (cd /tmp && /bin/bash "$PREV_UPDATES_DIR/$f" >> "$UPDATE_LOG" 2>&1)
                if [ "$?" -eq 0 ]; then
                    log_update UPDATE SUCCEEDED
                    if [ -e /tmp/reboot_required ] || [ "$ALWAYS_REBOOT" == "true" ]
                    then
                        log_update REBOOT REQUIRED
                        /sbin/shutdown -r now
                        exit 0
                    fi
                else
                    log_update UPDATE FAILED
                fi
            done
        fi

        count=$(ls -l ./*.rpm 2>/dev/null | wc -l)
        if [ "$count" != 0 ]; then
            # Get files from oldest to newest
            files=$(ls -rt ./*.rpm)
            for f in $files; do
                log_update "$f"
                mv "$f" "$PREV_UPDATES_DIR"
                echo "Installing \"$f\" ..."
                (rpm -ivh "$PREV_UPDATES_DIR/$f" >> "$UPDATE_LOG" 2>&1)
                if [ "$?" -eq 0 ]; then
                    log_update UPDATE SUCCEEDED
                    if [ -e /tmp/reboot_required ] || [ "$ALWAYS_REBOOT" == "true" ]
                    then
                        log_update REBOOT REQUIRED
                        /sbin/shutdown -r now
                        exit 0
                    fi
                else
                    log_update UPDATE FAILED
                fi
            done
        fi

        count=$(ls -l ./*.ipk 2>/dev/null | wc -l)
        if [ "$count" != 0 ]; then
            # Get files from oldest to newest
            files=$(ls -rt ./*.ipk)
            new_files=""
            # Get list of ipks that are newer than current version
            for f in $files; do
                mv "$f" "$PREV_UPDATES_DIR"
                if check_version "$PREV_UPDATES_DIR/$f"
                then
                    log_update "Package $f already up to date"
                else
                    new_files="$new_files $PREV_UPDATES_DIR/$f"
                fi
            done

            if [ -n "$new_files" ]; then
                log_update "Installing $new_files:"
                # shellcheck disable=SC2086
                (opkg install $new_files >> "$UPDATE_LOG" 2>&1)
                if [ $? -eq 0 ]; then
                    log_update UPDATE SUCCEEDED
                    if [ -e /tmp/reboot_required ] || [ "$ALWAYS_REBOOT" == "true" ]
                    then
                        log_update REBOOT REQUIRED
                        /sbin/shutdown -r now
                        exit 0
                    fi
                else
                    log_update UPDATE FAILED
                fi
            fi
        fi

    else
        echo "No Updates to Install"
    fi
    ;;

stop)
    ;;

restart)
    $0 stop
    $0 start
    ;;

*)
    echo "Usage: $0 {start|stop|restart}"
    exit 1
    ;;

esac

echo exiting with return code 0
exit 0
