#!/bin/sh
# 
# Factory Config Helper Functions
# 
# Compatible with Factory Config version 1.2.
# Used with MitySOM-5CSx and MitySOM-A10S.
# 

# Default EEPROM location.
EEPROM=${EEPROM:-/sys/bus/i2c/devices/i2c-0/0-0050/eeprom}

# Factory config magic and version numbers as of 01/21/2019.
FC_MAGIC="012C0138"
FC_VER="00010002"

# 
# Get the factory config Magic Number of the unit.
# Bytes 0-3, unsigned int (Little Endian)
# 
FCmagic() {
  od -A n -j 0 -N 4 -t x4 "$EEPROM" | tr -d ' ' | tr '[a-z]' '[A-Z]'
}

# 
# Get the factory config Version Number of the unit.
# Bytes 4-7, unsigned int (Little Endian)
# 
FCcfgver() {
  od -A n -j 4 -N 4 -t x4 "$EEPROM" | tr -d ' '
}

# 
# Get the MAC Address of the unit.
# Bytes 8-13, unsigned short int[3] (Little Endian)
# 
# e.g. 0xFFC4 0x70BC 0xD90F -> C4:FF:BC:70:0F:D9
# 
FCmacaddr() {
  od -A n -j 8 -N 6 -x "$EEPROM" | tr '[a-z]' '[A-Z]' | \
      sed 's/ \(..\)\(..\) \(..\)\(..\) \(..\)\(..\)/\2:\1:\4:\3:\6:\5/'
}

# 
# Get the Serial Number of the unit (Bytes 24-27).
# Bytes 24-27, unsigned int (Little Endian)
# 
FCserialnum() {
  od -A n -j 24 -N 4 -i "$EEPROM" | tr -d ' '
}

# 
# Get the Model Number of the unit.
# Bytes 28-59, char[32] (ASCII null-terminated/null-padded string)
# 
FCpartnum() {
  od -A n -j 28 -N 32 -a -w32 "$EEPROM" | tr -d ' ' | sed 's/nul.*//g'
}

# 
# Check if factory config is valid.
# 
# Returns
#   0 - Factory config is valid
#   1 - Factory config is invalid. (corrupt magic or version)
# 
FCisvalid() {
  test "$(FCmagic)" = "$FC_MAGIC" -a "$(FCcfgver)" = "$FC_VER"
}

# 
# Checks the validity of the factory config. This function is implemented
# within a subshell, so the environment will not be modified. Fields will
# not be checked if set to an empty string but will still be printed.
# 
# e.g. FCcheck "$model" "" "$mac" # don't check the serial number
# 
# Parameters
#   $1 - Expected model number
#   $2 - Expected serial number
#   $3 - Expected MAC address
# Returns (bit field)
#   0 - Factory config is valid and all fields match.
#   1 - Factory config is invalid. (corrupt magic or version)
#   2 - Model number mismatch.
#   4 - Serial number mismatch.
#   8 - MAC address mismatch.
# 
FCcheck() (
  printf "Magic   = $(FCmagic)\n"
  printf "Version = $(FCcfgver)\n"
  
  if FCisvalid; then
    model="$(FCpartnum)"
    serial="$(FCserialnum)"
    mac="$(FCmacaddr)"
    printf "Model Number  = $model\n"
    printf "Serial Number = $serial\n"
    printf "MAC Address   = $mac\n"
    rc=0
    [ "$1" -a "$1" != "$model"  ] && rc=$(($rc | 2))
    [ "$2" -a "$2" != "$serial" ] && rc=$(($rc | 4))
    [ "$3" -a "$3" != "$mac"    ] && rc=$(($rc | 8))
    return $rc
  else
    printf "FC Failed validation\n"
    return 1
  fi
)

# 
# (LEGACY)
# 
# Checks the validity of the factory config. This function should be called
# from within a subshell because it calls 'exit'. This is especially
# important if the caller is attatched to the test fixture GUI; in which case,
# the exit signal should be trapped in order to print the appropriate
# "End-Of-Test" sequence before death.
# 
# Returns
#   0 - Factory config is valid.
# Exits
#   1 - Factory config is invalid. (corrupt magic or version)
# 
FCTestAll() {
  printf "Magic   = $(FCmagic)\n"
  printf "Version = $(FCcfgver)\n"
  
  if FCisvalid; then
    printf "Serial  = $(FCserialnum)\n"
    printf "MACAddr = $(FCmacaddr)\n"
    printf "PartNum = $(FCpartnum)\n"
    return 0
  else
    printf "FC Failed validation\n"
    exit 1
  fi
}

